
io.stdout:setvbuf"no"

-- libraries
Gamestate = require "lib.hump.gamestate"
Timer = require "lib.hump.timer"
anim8 = require "lib.anim8"

-- helpers
require "window" 
require "resources"
require "physics"
require "input"

-- game objects; "actors"
require "player"
require "tracks"
require "gold"
require "obstacles"
require "scenery"

-- states
require "game"

-- global constants / reference
GAME_TIME = 0
GRAVITY = 800
MOVE_SPEED = 150
JUMP_SPEED = 120
JUMP_TIME = 0.3
TRACK_SPEED = 230
GOLD_BOUNCE_FACTOR = 100
BG_DARKNESS = 150

-- lovestuff
function love.load()
	-- convenience iteration
	map = function(list, func, ...)
		for i=1, #list do
			func(list[i], ...)
		end
	end

	-- same as map, returns true to keep, false to remove
	sweep = function(list, func, ...)
		for i=#list, 1, -1 do
			if not func(list[i], ...) then
				table.remove(list, i)
			end
		end
	end

	-- used for finding an element with specific criteria
	find = function(list, func, ...)
		for i=1, #list do
			local result = func(list[i])
			if result then
				return result
			end
		end
	end

	-- use with above function, makes a function to find a specific object
	searchFor = function(value)
		return function(i)
			return i == value
		end
	end

	-- convenience instantation
	new = function(parent, ...)
		local instance = setmetatable({}, { __index = parent })
		instance.__parent = parent
		parent.init(instance, ...)
		return instance
	end

	-- reverse the result of a function
	inverse = function(func)
		return function(...)
			return not func(...)
		end
	end

	math.randomseed(os.time())
	Window:init("Mine Jumper", 200, 150, 4)
	Resources:load()
	Scenery:load()
	Player:load()
	Gamestate.registerEvents()
	Gamestate.switch(Game)
end

function love.update(dt)
	GAME_TIME = GAME_TIME + dt
	Window:update(dt)
end

function love.keypressed(k)
	if k == 'escape' then
		love.event.quit()
	end
end

function love.draw()
	-- for i=1, 30000000 do end
end

