Physics = {}

-- get the sign of a number
local function sign(n)
	return n > 0 and 1 or n < 0 and -1 or 0
end

-- fast check that returns true if 2 boxes are intersecting
-- taken from bump.lua
local function boxesIntersect(l1,t1,w1,h1, l2,t2,w2,h2)
  return l1 <= l2+w2 and l1+w1 >= l2 and t1 <= t2+h2 and t1+h1 >= t2
end

-- returns the area & minimum displacement vector given two intersecting boxes
-- taken from bump.lua
local function getDisplacement(l1,t1,w1,h1,c1x,c1y, l2,t2,w2,h2,c2x,c2y)
  local dx = l2 - l1 + (c1x < c2x and -w1 or w2)
  local dy = t2 - t1 + (c1y < c2y and -h1 or h2)

  if math.abs(dx) < math.abs(dy) then return dx, 0 end
  return 0, dy
end

function Physics.implement(self, x, y, width, height)
	self.x = self.x or x or 0
	self.y = self.y or y or 0
	self.width = self.width or width or 16
	self.height = self.height or height or 16
	self.xvel = 0
	self.yvel = 0
	self.oldState = {}
end

function Physics.update(self, dt)
	self.oldState = {}
	for i,v in pairs(self) do
		self.oldState[i] = v
	end

	local deltayvel = GRAVITY * (self.weight or 1) * dt
	-- self.yvel = self.yvel + deltayvel / 2

	self.x = self.x + self.xvel * dt
	self.y = self.y + self.yvel * dt

	self.yvel = self.yvel + deltayvel
end

function Physics.colliding(self, other)
	if boxesIntersect(self.x, self.y, self.width, self.height, other.x, other.y, other.width, other.height) then
		return getDisplacement(
			self.x, self.y, self.width, self.height,
			self.x + self.width/2, self.y + self.height / 2,
			other.x, other.y, other.width, other.height,
			other.x + other.width/2, other.y + other.height / 2)
	end
end

function Physics.resolve(self, dx, dy)
	self.x = self.x + dx
	self.y = self.y + dy
	if self.xvel and dx ~= 0 and sign(dx) ~= sign(self.xvel) then
		self.xvel = 0
	end
	if self.yvel and dy ~= 0 and sign(dy) ~= sign(self.yvel) then
		self.yvel = 0
	end
end

function Physics.resolvePassThrough(self, other, dx, dy)
	if dy < 0 and self.yvel >= 0 and self.oldState.y + self.height <= other.y + 4 then
		Physics.resolve(self, dx, dy)
		return true
	end
	return false
end
